# -*- coding: utf-8 -*-
# Q12: Verschlüsselung 
# RSA
#
# Aufgabe:
# Finde zwei große Primzahlen p, q und multipliziere diese.
#   z.B. primes(1000)[10], primes(1000)[90]
#        p=31 ; q=467
# Berechne n durch p*q:
#       n = 14477
# Berechne phi(n) mit (p-1)(q-1)
#       phi(n) = 13980
# Finde d und e, so dass d*e kongruent 1%phi(n)
#       primfakt(1+13980)
#       d=341, e=41
# Gib n und e als Schlüssel aus, behalte d und verwerfe phi(n), p, q
#       öffentl. Schlüssel = (n,e)
# Verschlüsseln durch schnelles mod. Potenzieren: code = original**e % n
#       modpot(original,e,n)
# Entschlüsseln durch schnelles mod. Pot: original = code**d % n
#       modpot(code,d,n)

from string import upper
import sys
sys.maxrecursionlimit = 1000000

# Hilfsfunktion "größter gemeinsamer Teiler"
def ggt(a,b):
    while b:
        return ggt(b,a%b)
    return a


# schneller Primzahlgenerator: liefert Liste
def primes(n): 
	if n==2: return [2]
	elif n<2: return []
	s=range(3,n+1,2)
	mroot = n ** 0.5
	half=(n+1)/2-1
	i=0
	m=3
	while m <= mroot:
		if s[i]:
			j=(m*m-3)/2
			s[j]=0
			while j<half:
				s[j]=0
				j+=m
		i=i+1
		m=2*i+3
	return [2]+[x for x in s if x]

# Primzahltest nach Miller-Rabin
def primMR(n):
    "testet Primzahleigenschaft für n < 341.550.071.728.321"
    primzahlen=[2, 3, 5, 7, 11, 13, 17]
    for i in primzahlen:
        if i**3%n == 1%n:
            return True

# Primfaktorenzerlegung durch Probedivision
from math import sqrt
def primfakt(n):
    faktoren=[]
    for i in primes(n):
        while not n%i:
            n=n/i
            faktoren.append(i)
    return faktoren

def modpot(x,y,n):
    '''Schnelles (modulares) Potenzieren x^y (mod n)
    nach der Methode "square and multiply".
    Bei jedem Schritt wird modulo n reduziert!'''
    pot = 1
    while y > 0:
        if y % 2 == 1:          # Falls Exponent ungerade:
            pot = (pot * x) % n # Multiplizieren!
            y = y - 1           # Nun ist der Exponent wieder gerade
        else:
            x = (x * x) % n     # Quadrieren und
            y = y / 2           # Exponent halbieren
    return pot                  

def text2zahl(text):
    "gib Monsterzahl aus String zurück"
    zahl=""
    for i in text:                  # hole immer ein Zeichen
        if i in [' ','_']:          # falls Leerzeichen oder Unterstrich
            zahl+="00"              # setze 00
        else:
            wert = ord(upper(i))-64 # a->A soll 01 sein
            zahl+=str(wert).zfill(2)# fülle vorn mit 0 auf
    return int(zahl)

def zahl2text(zahl):
    "gibt Text aus Monsterzahl zurück"
    original=""
    while zahl:
        original=chr(zahl%100+64)+original
        zahl=zahl/100
    return original
        
        

