# Erzeuger-Verbraucher-Lösung mit Semaphore
# 

import time
from threading import *
from random import randint, shuffle
from visual import *

class Lager:
    "Semaphoren-kontrolliertes Lager mit Mehrfachzugriff, Kapazität und belegung" 
    def __init__(self,kapazitaet,belegung,maxZugriff):
        self.kapazitaet = kapazitaet
        self.belegung     = belegung
        self.zugriff    = Semaphore(maxZugriff)
        # Spielfigur
        self.halle = cylinder(opacity=0.5, y=-.5, color=color.yellow, axis=(0,.5,0), radius=3)
        # leere Lagerpositionen
        self.leerpos = [(-1,0,0),(1,0,0),(0,0,-1),(0,0,1),\
                        (-1,0,-1),(-1,0,1),(1,0,-1),(1,0,1),\
                        (0,0,0)]
        # besetzte Lagerpositionen
        self.vollpos = []
    

class Erzeuger(Thread):
    "modelliert Produzent für Lagerwaren"

    # Klassenvariable für Positionsbestimmung auf z-Achse
    posz = -3

    def __init__(self,lager,warteZeit=1):
        "Erzeuger erzeugen"
        Thread.__init__(self)
        self.lager = lager
        self.warteZeit=warteZeit
        # Spielfigur, die in der z-Achse verschoben wird ----------------------
        self.startpos=(-5,0,Erzeuger.posz)
        Erzeuger.posz+=2
        self.gestalt = box(pos=self.startpos,color=color.blue)
        # ---------------------------------------------------------------------
    def run(self):
        "Erzeuger starten"
        while 1:                            # workaholic...
            if self.lager.belegung < self.lager.kapazitaet: # ...ist noch Platz im Lager?
                self.lager.zugriff.acquire()    # Zugriff auf Lager anfordern
                wz = randint(1,self.warteZeit)  # Betriebspause per random im Intervall [1-warteZeit] ermitteln
                # zufällige Auswahl eines leeren Lagerplatzes ---------------------
                shuffle(self.lager.leerpos)
                self.lagerpos = self.lager.leerpos.pop()
                self.lager.vollpos.append(sphere(pos=self.startpos, color=color.red, radius=.5))   # Ware mit Spur erzeugen ...
                self.lager.vollpos[-1].pos.y+=.5
                time.sleep(0.3)
                self.lager.vollpos[-1].pos = (-2.5,3,0)                        # ... via Scheitelpunkt ...
                time.sleep(0.3)
                self.lager.vollpos[-1].pos = self.lagerpos                  # ... und ablegen
                time.sleep(0.3)
                # -----------------------------------------------------------------
                self.lager.belegung+=1        # dann erst Produkt erzeugen
                print "%s\t%ss\t -----> %s"%(currentThread().name,wz,self.lager.belegung)
            self.lager.zugriff.release()    # Lager wieder freigeben
            time.sleep(wz)                  # warten, damit andere Erzeuger/Verbraucher tätig werden können


class Verbraucher(Thread):
    "modelliert Verbraucher für Lagerwaren"

    # Klassenvariable für Positionsbestimmung auf z-Achse
    posz = 3
  
    def __init__(self,lager,warteZeit=1):
        "Verbraucher erzeugen"
        Thread.__init__(self)
        self.lager = lager
        self.warteZeit=warteZeit
        # Spielfigur -------------------------------------------------------------
        self.startpos=(5,0,Verbraucher.posz)
        Verbraucher.posz -= 2
        self.gestalt = box(pos=self.startpos, color=color.green)

    def run(self):
        "Verbraucher starten"
        while 1:                            # immer auf Abruf...
            wz = randint(1,self.warteZeit)  # warteZeit per random berechnen
            if self.lager.belegung:           # Verklemmungsschutz: ist Lager gefüllt?
                self.lager.zugriff.acquire()    # Zugriff auf Lager anfordern
                # zufällige Auswahl eines besetzten Lagerplatzes ------------------
                shuffle(self.lager.vollpos)
                self.lagerpos = self.lager.vollpos.pop()
                time.sleep(0.3)
                self.lagerpos.pos = (2.5,3,0)                        # ... via Scheitelpunkt ...
                time.sleep(0.3)
                self.lagerpos.pos = self.startpos
                self.lagerpos.y +=0.5
                time.sleep(0.3)
                self.lagerpos.visible = False
                self.lager.leerpos.append(self.lagerpos.pos) # freien Lagerplatz zurückgeben
                del(self.lagerpos)
                # -----------------------------------------------------------------
                self.lager.belegung-=1        # dann erst Produkt verbrauchen
                print "\t\t\t\t%s ----->\t%ss\t%s"%(self.lager.belegung,wz,currentThread().name)
            self.lager.zugriff.release()    # Lager freigeben
            time.sleep(wz)                  # warten, damit andere Erzeuger/Verbraucher tätig werden können
        
        
        
if __name__=="__main__":

    bauteillager=Lager(9,0,2)              # Kapazität =<9, Start 0, binäre Semaphore
    erz1=Erzeuger(bauteillager,4) 
    erz2=Erzeuger(bauteillager,3)
    erz3=Erzeuger(bauteillager,5)
    ver1=Verbraucher(bauteillager,6)
    ver2=Verbraucher(bauteillager,5)
    erz1.start()
    erz2.start()
    ver1.start()
    ver2.start()
    
    while 1:
        pass
