# -*- coding: utf-8 -*-
# Grenzen der Berechenbarkeit
# Alternative Fibonacci-Algorithmen im Vergleich

MAXn = 50

import time                     # Zeitmodul
from visual.graph import *      # Alle Klassen zum Graph-Modul laden
from math import log	        # Logarithmus aus Mathe-Modul laden

def fibo(n):
    "Fibonacci-Funktion rekursiv"
    if n==0 or n==1:
        return n
    else:
        return fibo(n-1)+fibo(n-2)

def fibit(n):
    "Fibonacci-Funktion iterativ"
    a, b = 0, 1
    for i in range(n):
        a, b = b, a + b
    return a

def fibbin(n):
    "Fibonacci-Funktion nach der Formel von Binet: schnell, ungenau"
    # Seitenverhältnis "Goldener Schnitt"
    phi=(1+5**0.5)/2
    return int(round((phi**n - (1-phi)**n)/5**0.5))

def fibewd(n):
    "Fibonacci-Funktion nach der E.W. Dijkstra-Formel: schnell, genau"
    fibs={0:0,1:1}
    if n in fibs: return fibs[n]
    if n%2==0:
        fibs[n] = ((2*fibewd((n/2)-1)) + fibewd(n/2)) * fibewd(n/2)
    else:
        fibs[n] = (fibewd((n-1)/2)**2) + (fibewd((n+1)/2)**2)
    return fibs[n]


# Fenster für grafische Ausgabe
fibograph = gdisplay(width=1000,height=700,\
                     title="Laufzeitanalyse Fibonacci-Funktion",\
                     xtitle="n", ytitle="t[s]",xmin=0, xmax=MAXn,\
                     background=color.white,foreground=color.black)
# x-Achse nicht autoskaliert
fibograph.autoscale[0]=0
# Funktionenschar
func = [gcurve(color=color.blue),gcurve(color=color.red),\
        gcurve(color=color.green),gcurve(color=color.orange)]#, gcurve(color=color.magenta)]

aufruf=("fibbin(i)","fibit(i)","fibewd(i)","fibo(i)")

for i in range(MAXn):
    for j in range(len(func)):
        anfang = time.time()
        ergebnis = eval(aufruf[j])
        ende = time.time()
        laufzeit = ende-anfang
        #print "%3i %10f %10i"%(i, laufzeit, ergebnis)
        func[j].plot(pos=(i,laufzeit))
    print i


