# -*- coding: cp1252 -*-
# Modellierung einer 3D-Verkehrsampel
# SH, 2009-10-19

from visual import *                                # importiere alles aus Modul 'visual'
from time   import sleep                            # importiere 'sleep' aus Modul 'time'
import threading                                    # importiere Modul 'threading'
import winsound                                     # importiere Modul 'winsound'

# Konstanten
LAMPEAUS = (0.2,0.2,0.2)                            # Rot-gruen-Blau-Muster: dunkelgrau
GEHAEUSE = (0.2,0.3,0.25)                           # dunkelgruen
SUED     = ( 1, 0, 0)                               # +x - Richtung (rechts)
NORD     = (-1, 0, 0)                               # -x - Richtung (links)
WEST     = ( 0, 0, 1)                               # +z - Richtung (zum Betrachter)
OST      = ( 0, 0,-1)                               # -z - Richtung (vom Betrachter weg)
STRASSE  = (0.1,0.1,0.1)                            # dunkeldunkelgrau
FELD     = (0.4, 0.5, 0.2)                          # graugruen
RADIUS   = 500                                      # Groesse der gruenflaeche
HAUT     = (1,.6,.4)                                # Hautfarbe
HAAR     = (.5,.4,.3)                               # Haarfarbe
# Fenstereinstellungen
scene.title = "Ampelschaltung Klasse 10a"           # Fenstertitel
scene.x = 10                                        # Fensterabstand links
scene.y = 10                                        # Fensterabstand oben
scene.width = 1000                                  # Fensterbreite
scene.height = 400                                  # Fensterhoehe
scene.forward = vector(-0.83, -0.22, 0.5)*4         # Blickwinkel
scene.range = 80                                    # Kameraabstand
scene.show_rendertime = True                        # Anzeige Bildaufbau
scene.fullscreen = False                            # Vollbildmodus?
#scene.userzoom = False                              # Zoomen erlaubt?
#scene.userspin = False                              # Blickwinkel aendern erlaubt?

class Lampe(sphere):
    """Klasse modelliert Licht f. Ampel"""
    def __init__(self, farbe=(1,0,0), pos=(0,2,0),zustand=False, frame=None):
        "Konstruktor fr Ampellicht"
        # Konstruktoraufruf Oberklasse
        sphere.__init__(self, color=farbe, pos=pos, frame=frame)
        self.farbe = farbe                  # Farbe             
        self.zustand = zustand              # True = "an", False = "aus"
        self.material = materials.emissive  # "leuchtende" Oberflaeche
        if zustand:
            self.color = farbe
        else:
            self.color = LAMPEAUS

    def an(self):
        "schaltet Lampe an"
        self.color = self.farbe
        self.zustand = True             # Zustand anpassen

    def aus(self):
        "schaltet Lampe aus"
        self.color = LAMPEAUS
        self.zustand = False            # Zustand anpassen

    def schalten(self):
        "veraendert Lampenzustand"
        if self.zustand:                # falls zustand==True (heisst 'Lampe an'):
            self.aus()                  # Lampe aus
        else:                           # sonst:
            self.an()                   # Lampe an 

    def blinken(self, anzahl=10):
        "an- und auschalten entsprechend 'anzahl'"
        while anzahl:               
            self.an()                   # Lampe an
            sleep(1)                    # 1s warten
            self.aus()                  # Lampe aus
            sleep(1)                    # 1s warten
            anzahl = anzahl-1           # um eins verringern


class Ampel(threading.Thread, frame):
    """Container-Klasse Ampel mit 3 Lampen"""
    def __init__(self, pos=(0,0,0), axis=(1,0,0)):
        "Konstruktor 3D-Verkehrsampel"
        threading.Thread.__init__(self)                             # Konstruktor Oberklasse
        frame.__init__(self, pos=pos, axis=axis)                    # Konstruktor Oberklasse
        self.rot   = Lampe(frame = self,farbe=(1,0,0),pos  =(0,7,1))# Rot
        self.gelb  = Lampe(frame = self,farbe=(1,1,0),pos = (0,5,1))# Gelb
        self.gruen  = Lampe(frame = self,farbe=(0,1,0),pos = (0,3,1))# gruen
        self.kasten= box(frame=self, pos=(0,5,0), height=7, width=3, length=3, \
                         color=GEHAEUSE, opacity=0.9)               # Ampelkasten
        self.stange= cylinder(color=GEHAEUSE, frame=self, pos=(0,-10,0), \
                              radius=0.7, axis=(0,12,0))            # Ampelstange
        self.modus = "aus"                                          # "aus"/"an"/"standby"
        
    def stoerung(self,anzahl=1):
        "laesst Gelblicht entspr. 'anzahl' blinken"
        self.rot.aus()
        self.gelb.an()
        self.gruen.aus()
        self.gelb.blinken(anzahl)

    def run(self):
        "mit 'Thread.start()' wird diese Methode aktiviert"
        while True:                                             # Endloswiederholung
            if self.modus == "an":                              # falls Ampel an:
                self.schalten()                                 # Ampel schalten
            elif self.modus == "standby":                       # falls Ampel stand-by:
                self.stoerung()                                  # Ampel mit Gelblichtblinken
            
    def schalten(self):
        "rot->rotgelb->gruen-Phasen wechseln"
        self.modus = "an"
        self.rot.aus()
        self.gelb.an()
        self.gruen.aus()
        sleep(2)
        self.rot.an()
        self.gelb.aus()
        sleep(5)
        self.gelb.an()
        sleep(2)
        self.rot.aus()
        self.gelb.aus()
        self.gruen.an()
        sleep(2)
        self.gruen.aus()
        self.modus = "aus"

       

class FuAmpel(threading.Thread, frame):
    """Container-Klasse Ampel mit 3 Lampen"""
    def __init__(self, pos=(0,0,0), axis=(1,0,0)):
        "Konstruktor 3D-Verkehrsampel"
        threading.Thread.__init__(self)                             # Konstruktor Oberklasse
        frame.__init__(self, pos=pos, axis=axis)                    # Konstruktor Oberklasse
        self.rot   = Lampe(frame = self,farbe=(1,0,0),pos  =(0,5,1))# Rot
        self.gruen  = Lampe(frame = self,farbe=(0,1,0),pos = (0,3,1))# gruen
        self.kasten= box(frame=self, pos=(0,4,0), height=5, width=3, length=3, \
                         color=GEHAEUSE, opacity=0.9)               # Ampelkasten
        self.stange= cylinder(color=GEHAEUSE, frame=self, pos=(0,-10,0), \
                              radius=0.7, axis=(0,12,0))            # Ampelstange
        self.taster= box(color=color.orange, frame=self, \
                         pos=(0,-3,-1), size=(1,1.5,1))              # Taster fr Bedienung
        self.modus = "aus"                                          # "aus"/"an"/"standby"
        
    def run(self):
        "mit 'Thread.start()' wird diese Methode aktiviert"
        while True:                                             # Endloswiederholung
            if self.modus == "an":                              # falls Ampel an:
                self.schalten()                                 # Ampel schalten
            
    def schalten(self):
        "rot (2) -> gruen (5) -> rot"
        self.modus = "an"
        self.rot.an()
        self.gruen.aus()
        sleep(3)
        self.rot.aus()
        self.gruen.an()
        sleep(4)
        self.rot.an()
        self.gruen.aus()
        sleep(4)
        self.rot.aus()      
        self.modus = "aus"


class Auto(threading.Thread, frame):
    """Container-Klasse Auto"""
    def __init__(self, pos=(0,0,0), richtung=SUED, farbe=(1,0,1), speed=0.003, ampel=None):
        "Konstruktor 3D-Verkehrsampel"
        threading.Thread.__init__(self)                             # Konstruktor Oberklasse
        frame.__init__(self, pos=pos)                    # Konstruktor Oberklasse
        self.karosserie = box(frame=self, color=farbe, pos=(0,0,0), size=(5,1.5,3))
        self.kabine = box(frame=self,  color=(0.5,0.5,1), pos=(0,1,0), size=(4,2,2.9), opacity=.5)
        self.rad_lv = ring(frame=self, pos=(1,-1,1.5)  , radius=.5, thickness=0.5, color=color.gray(0.3), axis=(0,0,1))
        self.rad_rv = ring(frame=self, pos=(1,-1,-1.5) , radius=.5, thickness=0.5, color=color.gray(0.3), axis=(0,0,1))
        self.rad_lh = ring(frame=self, pos=(-1,-1,1.5) , radius=.5, thickness=0.5, color=color.gray(0.3), axis=(0,0,1))
        self.rad_rh = ring(frame=self, pos=(-1,-1,-1.5), radius=.5, thickness=0.5, color=color.gray(0.3), axis=(0,0,1))
        self.speed = speed
        self.richtung = richtung
        self.ampel = ampel

    def run(self):
        "mit 'Thread.start()' wird diese Methode aktiviert"
        while True:                                             # Endloswiederholung
            if self.ampel.rot.zustand \
               and mag(self.pos-self.ampel.pos) < 15:           # Ampel rot?
                pass                                            # ...anhalten!
            
            elif self.ampel.gelb.zustand \
               and mag(self.pos-self.ampel.pos) < 50:           # Ampel gelb?
                self.pos = self.pos - 0.5*self.speed*vector(self.richtung)
                                                                # ...halbe Fahrt!
            else:                                               # Ampel gruen oder aus
                self.pos = self.pos - self.speed*vector(self.richtung)# ...Vollgas!
                
            if self.x < -RADIUS:                                 # Orbit verlassen?
                self.x = RADIUS                                 # neue Position
            elif self.x > RADIUS:                               # Orbit verlassen?
                self.x = -RADIUS                                # neue Position
                

class Person(threading.Thread, frame):
    """ Container-Klasse Person """
    def __init__(self, spos=(0,0,0), axis=WEST, ampel=None):
        "Konstruktor 3D-Verkehrsampel"
        threading.Thread.__init__(self)                             # Konstruktor Oberklasse
        frame.__init__(self, pos=spos, axis=axis)                    # Konstruktor Oberklasse
        self.spos  = spos
        self.koerper= cylinder(frame=self,color=color.blue,pos=(0,7,0),axis=(0,2,0),radius=1)    # Koerper
        self.nase  =     cone(frame=self,color=HAUT,pos=(0.9,9.8,0)  ,axis=(0,.6,0), radius=.3) # Nase
        self.beinl = cylinder(frame=self,color=HAUT,pos=(0,5, .5)    ,axis=(0,2,0),radius=.5)   # Bein                                          # "aus"/"an"/"standby"
        self.beinr = cylinder(frame=self,color=HAUT,pos=(0,5,-.5)    ,axis=(0,2,0),radius=.5)   # Bein                                          # "aus"/"an"/"standby"
        self.kopf  =   sphere(frame=self,color=HAUT,pos=(0,10,0)     ,radius=1)                 # Kopf                                          # "aus"/"an"/"standby"
        self.haare =   sphere(frame=self,color=HAAR,pos=(-.1,10.2,0) ,radius=1)                 # Haar
        self.arml  = cylinder(frame=self,color=HAUT,pos=(0,6.9, 1.3) ,axis=(0,2,-.6),radius=.25)# Bein                                          # "aus"/"an"/"standby"
        self.armr  = cylinder(frame=self,color=HAUT,pos=(0,6.9,-1.3) ,axis=(0,2,0.6),radius=.25)# Bein                                          # "aus"/"an"/"standby"
        self.ampel = ampel
            
    def run(self):
        """ Fortbewegung bis Ziel"""
        while True:
            if not self.ampel.gruen.zustand:
                # Bewegung von Armen und Beinen...
                pass
                #self.pos = self.spos


if __name__ == "__main__":

    # Himmel und Hintergrundbeleuchtung
    scene.background= (0.5,0.6,0.8)
    scene.ambient   = color.gray(0.9)

    # Untergrundgestaltung ohne Autoskalierung
    scene.autoscale = 0
    wiese           = cylinder(pos=(0,-10,0), radius=RADIUS, axis=(0,1,0), color=FELD)
    strassenGrenze   = box(pos=(  0,-9.1,7.5), size=(2*RADIUS,1,10.5))
    strassenflaeche   = box(pos=(  0,  -9,7.5), size=(2*RADIUS, 1, 10), color=STRASSE)
    haltestrichOst  = box(pos=( 10,-8.9,  5), size=(1,1,5))
    haltestrichWest = box(pos=(-10,-8.9, 10), size=(1,1,5))
    
    # Ampeln erstellen und starten
    a1 = Ampel(pos=(5,0,0),axis=OST)
    a1.start()
    a2 = Ampel(pos=(-5,0,15),axis=WEST)
    a2.start()
    f1 = FuAmpel(axis=SUED)
    f1.start()
    f2 = FuAmpel(pos=(0,0,15), axis=NORD)
    f2.start()

    # Fahrzeuge erstellen und starten
    c1 = Auto(pos=(15,-6.5,5), ampel=a1)
    c1.start()
    c2 = Auto(pos=(-15,-6.5,10),farbe=(1,.5,0),ampel=a2,richtung=NORD,speed=0.004)
    c2.start()

    # Person aufstellen
    p1=Person(spos=(-3,-13.5,0), axis=WEST, ampel=f1)
    p1.start()
    
    # Endloswiederholung mit Maus- und Tastaturabfrage
    dwinkel = 0.001                      # 
    while True:
        rate(25)                            # 25 Bilder pro Sekunde reichen
        scene.forward = scene.forward.rotate(angle=dwinkel, axis=(0,1,0.1))
        if scene.mouse.events:              # ...Mausereignis?
            mEvent = scene.mouse.getevent() # speichern in mEvent
            if mEvent.pick \
               == f1.taster or f2.taster:   # falls Taster unter dem Mauszeiger:
                winsound.MessageBeep(-1)    # Piepen auf Windows-Rechnern
                a1.modus = "an"             # schaltet alle Ampeln an
                a2.modus = "an"
                f1.modus = "an"
                f2.modus = "an"
                
        if scene.kb.keys:                   # ...Tastaturereignis?
            kEvent = scene.kb.getkey()      # speichert Tastendruck
            if   kEvent == "+":             # Plustaste gedrckt?
                dwinkel = dwinkel+0.001     # Beschleunigung Rechtsdrehung 
            elif kEvent == "-":             # Minustaste gedrckt?
                dwinkel = dwinkel-0.001     # Beschleunigung Linksdrehung
            elif kEvent == " ":              # Leertaste gedrckt?
                dwinkel = 0                 # Winkelbeschleunigung aus
            elif kEvent == "up" and p1.z < 13:            # Pfeil-hoch-Taste gedrckt?
                p1.pos = p1.pos + vector(p1.axis)*0.2
                if p1.z == 13:
                    p1.rotate(angle=pi/2,axis=(0,1,0))
                    print(p1.z)

            elif kEvent == "down" and p1.z>1:          # Pfeil-runter-Taste gedrckt?
                p1.pos = p1.pos - vector(p1.axis)*0.2
            
                
